﻿Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Net
Imports System.IO
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media.Animation
Imports System.Windows.Shapes

Imports Microsoft.Xna.Framework
Imports Microsoft.Xna.Framework.Graphics
Imports System.Windows.Xna
Imports System.Windows.Media.Imaging


Public Structure VertexPositionColor
    Public Position As Vector3
    Public Color As Color


    Public Sub New(ByVal position As Vector3, ByVal color As Color)
        Me.Position = position
        Me.Color = color
    End Sub

    Public Shared ReadOnly VertexDeclaration As New VertexDeclaration(New VertexElement(0, VertexElementFormat.Vector3, VertexElementUsage.Position, 0), New VertexElement(12, VertexElementFormat.Color, VertexElementUsage.Color, 0))
End Structure

Partial Public Class MainPage
    Inherits UserControl
    ' init the 3D scene
    Private showPrimitives As Boolean
    ' resources
    Private vertexBuffer As VertexBuffer
    Private vertexShader As VertexShader
    Private pixelShader As PixelShader
    Private texture As Texture2D

    Private Shared ReadOnly resourceDevice As GraphicsDevice = System.Windows.Graphics.GraphicsDeviceManager.Current.GraphicsDevice

    '  Scene scene = new Scene();
    Private cubeXAxis As Boolean
    Private cubeYAxis As Boolean
    Private cubeZAxis As Boolean
    Private translateZ As Double
    Private rotateX As Double
    Private view As Matrix ' The view or camera transform
    Private _projection As Matrix ' The projection transform to convert 3D space to 2D screen space

    Public Sub New()
        InitializeComponent()
    End Sub

    Private Sub sliderX_ValueChanged(ByVal sender As Object, ByVal e As RoutedPropertyChangedEventArgs(Of Double))
        rotateX = e.NewValue
        ToolTipService.SetToolTip(sliderX, e.NewValue.ToString())

    End Sub
    Private Sub sliderZ_ValueChanged(ByVal sender As Object, ByVal e As RoutedPropertyChangedEventArgs(Of Double))
        translateZ = e.NewValue
        ToolTipService.SetToolTip(sliderZ, e.NewValue.ToString())

    End Sub
    Private Sub RotateX_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        cubeXAxis = Not cubeXAxis
        If cubeXAxis Then
            btnRotateX.Content = "Pause X"
        Else
            btnRotateX.Content = "Rotate X"
        End If

    End Sub
    Private Sub RotateY_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        cubeYAxis = Not cubeYAxis
        If cubeYAxis Then
            btnRotateY.Content = "Pause Y"
        Else
            btnRotateY.Content = "Rotate Y"
        End If

    End Sub
    Private Sub RotateZ_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        cubeZAxis = Not cubeZAxis
        If cubeZAxis Then
            btnRotateZ.Content = "Pause Z"
        Else
            btnRotateZ.Content = "Rotate Z"
        End If
    End Sub
    Private Sub Light_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        If BulbGradient.Visibility = System.Windows.Visibility.Collapsed Then
            BulbGradient.Visibility = System.Windows.Visibility.Visible
            bgImage.Visibility = System.Windows.Visibility.Collapsed
            btnLight.Content = "Background"
        Else
            BulbGradient.Visibility = System.Windows.Visibility.Collapsed
            bgImage.Visibility = System.Windows.Visibility.Visible
            btnLight.Content = "Light"
        End If
    End Sub
    Private Sub Reset_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        cubeXAxis = False
        cubeYAxis = False
        cubeZAxis = False
        btnRotateX.Content = "Rotate X"
        btnRotateY.Content = "Rotate Y"
        btnRotateZ.Content = "Rotate Z"
        sliderX.Value = 0
        sliderZ.Value = 0

    End Sub
    Private Sub Primitives_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        showPrimitives = Not showPrimitives
        OnLoad(Me, e)
    End Sub
    Private Sub OnSizeChanged(ByVal sender As Object, ByVal e As SizeChangedEventArgs)
        Dim surface As DrawingSurface = TryCast(sender, DrawingSurface)
        Dim sceneAspectRatio As Single = CSng(surface.ActualWidth) / CSng(surface.ActualHeight)
        _projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4, sceneAspectRatio, 1.0F, 100.0F)
    End Sub
    Private Sub OnDraw(ByVal sender As Object, ByVal args As DrawEventArgs)
        Dim position As Matrix = Matrix.Identity
        Dim scale As Matrix = Matrix.CreateScale(1.0F)
        Dim xf As Single = 0.0F
        Dim yf As Single = 0.0F
        Dim zf As Single = 0.0F

        If ((Not cubeXAxis)) AndAlso ((Not cubeYAxis)) AndAlso ((Not cubeZAxis)) Then ' first time angle
            If rotateX <> 0 Then
                xf = CSng(rotateX)
            Else
                xf = -0.65F
            End If
            yf = 0.0F
            zf = 0.0F
        End If
        If cubeXAxis Then
            xf = MathHelper.PiOver4 * CSng(args.TotalTime.TotalSeconds)
        End If
        If cubeYAxis Then
            yf = MathHelper.PiOver4 * CSng(args.TotalTime.TotalSeconds)
        End If
        If cubeZAxis Then
            zf = MathHelper.PiOver4 * CSng(args.TotalTime.TotalSeconds)
        End If

        Dim rotation As Matrix = Matrix.CreateFromYawPitchRoll(xf, yf, zf)

        Dim world As Matrix
        If translateZ <> 0 Then
            world = rotation * Matrix.CreateTranslation(0, 0, CSng(translateZ))
        Else
            world = scale * rotation * position
        End If

        ' calculate the final coordinates to pass to the shader
        Dim worldViewProjection As Matrix = world * view * _projection
        resourceDevice.Clear(ClearOptions.Target Or ClearOptions.DepthBuffer, New Microsoft.Xna.Framework.Color(0, 0, 0, 0), 10.0F, 0)
        ' setup vertex pipeline
        resourceDevice.SetVertexBuffer(vertexBuffer)
        resourceDevice.SetVertexShader(vertexShader)
        resourceDevice.SetVertexShaderConstantFloat4(0, worldViewProjection)

        resourceDevice.SetPixelShader(pixelShader)
        resourceDevice.Textures(0) = texture
        resourceDevice.DrawPrimitives(PrimitiveType.TriangleList, 0, 12)
        args.InvalidateSurface()
    End Sub

    Private Sub OnLoad(ByVal sender As Object, ByVal e As RoutedEventArgs)
        vertexBuffer = CreateCube()
        Dim shaderStream As Stream = Application.GetResourceStream(New Uri("Cube3d;component/shader/shader.vs", UriKind.Relative)).Stream
        vertexShader = vertexShader.FromStream(resourceDevice, shaderStream)

        Dim pixelStream As Stream = Application.GetResourceStream(New Uri("Cube3d;component/shader/shader.ps", UriKind.Relative)).Stream
        pixelShader = pixelShader.FromStream(resourceDevice, pixelStream)

        Dim imageStream As Stream = Application.GetResourceStream(New Uri("Cube3d;component/scene.jpg", UriKind.Relative)).Stream
        Dim image = New BitmapImage()
        image.SetSource(imageStream)
        texture = New Texture2D(resourceDevice, image.PixelWidth, image.PixelHeight, False, SurfaceFormat.Color)
        image.CopyTo(texture)

        Dim cameraPosition As New Vector3(0, 0, 5.0F) ' the camera's position
        Dim cameraTarget As Vector3 = Vector3.Zero ' the place the camera is looking (towards world origin)
        view = Matrix.CreateLookAt(cameraPosition, cameraTarget, Vector3.Up)
    End Sub
    Private Function CreateCube() As VertexBuffer
        ' cube vertices
        Dim vertexCollection = New VertexPositionColor(35) {}

        ' Front
        ' A-----------------B
        ' |                 |
        ' |                 |
        ' |                 |
        ' C-----------------D
        '  
        ' Back
        '    E-----------------F
        '    |                 |
        '    |                 |
        '    |                 |
        '    G-----------------H

        ' front coordinates
        Dim cubeA As New Vector3(-0.75F, 0.75F, 0.75F)
        Dim cubeB As New Vector3(0.75F, 0.75F, 0.75F)
        Dim cubeC As New Vector3(-0.75F, -0.75F, 0.75F)
        Dim cubeD As New Vector3(0.75F, -0.75F, 0.75F)
        ' back coordinates
        Dim cubeE As New Vector3(-0.75F, 0.75F, -0.75F)
        Dim cubeF As New Vector3(0.75F, 0.75F, -0.75F)
        Dim cubeG As New Vector3(-0.75F, -0.75F, -0.75F)
        Dim cubeH As New Vector3(0.75F, -0.75F, -0.75F)

        '  colors
        Dim cRed As Color = Color.FromNonPremultiplied(255, 0, 0, 156)
        Dim cGreen As Color = Color.FromNonPremultiplied(0, 255, 0, 156)
        Dim cBlue As Color = Color.FromNonPremultiplied(0, 0, 255, 156)
        Dim cYellow As Color = Color.FromNonPremultiplied(255, 255, 0, 156)
        Dim cBlack As Color = Color.FromNonPremultiplied(0, 0, 0, 156)
        Dim cWhite As Color = Color.FromNonPremultiplied(255, 255, 255, 156)

        If showPrimitives Then
            ' front 
            vertexCollection(0) = New VertexPositionColor(cubeA, cGreen)
            vertexCollection(1) = New VertexPositionColor(cubeB, cGreen)
            vertexCollection(2) = New VertexPositionColor(cubeC, cGreen)
            vertexCollection(3) = New VertexPositionColor(cubeB, cBlue)
            vertexCollection(4) = New VertexPositionColor(cubeD, cBlue)
            vertexCollection(5) = New VertexPositionColor(cubeC, cBlue)

            ' back  
            vertexCollection(6) = New VertexPositionColor(cubeG, cBlue)
            vertexCollection(7) = New VertexPositionColor(cubeF, cBlue)
            vertexCollection(8) = New VertexPositionColor(cubeE, cBlue)
            vertexCollection(9) = New VertexPositionColor(cubeH, cGreen)
            vertexCollection(10) = New VertexPositionColor(cubeF, cGreen)
            vertexCollection(11) = New VertexPositionColor(cubeG, cGreen)

            ' top 
            vertexCollection(12) = New VertexPositionColor(cubeE, cRed)
            vertexCollection(13) = New VertexPositionColor(cubeF, cRed)
            vertexCollection(14) = New VertexPositionColor(cubeA, cRed)
            vertexCollection(15) = New VertexPositionColor(cubeF, cYellow)
            vertexCollection(16) = New VertexPositionColor(cubeB, cYellow)
            vertexCollection(17) = New VertexPositionColor(cubeA, cYellow)

            ' bottom  
            vertexCollection(18) = New VertexPositionColor(cubeH, cRed)
            vertexCollection(19) = New VertexPositionColor(cubeG, cRed)
            vertexCollection(20) = New VertexPositionColor(cubeC, cRed)
            vertexCollection(21) = New VertexPositionColor(cubeD, cYellow)
            vertexCollection(22) = New VertexPositionColor(cubeH, cYellow)
            vertexCollection(23) = New VertexPositionColor(cubeC, cYellow)

            ' left 
            vertexCollection(24) = New VertexPositionColor(cubeC, cBlack)
            vertexCollection(25) = New VertexPositionColor(cubeG, cBlack)
            vertexCollection(26) = New VertexPositionColor(cubeA, cBlack)
            vertexCollection(27) = New VertexPositionColor(cubeA, cWhite)
            vertexCollection(28) = New VertexPositionColor(cubeG, cWhite)
            vertexCollection(29) = New VertexPositionColor(cubeE, cWhite)

            ' right  
            vertexCollection(30) = New VertexPositionColor(cubeH, cWhite)
            vertexCollection(31) = New VertexPositionColor(cubeD, cWhite)
            vertexCollection(32) = New VertexPositionColor(cubeB, cWhite)
            vertexCollection(33) = New VertexPositionColor(cubeH, cBlack)
            vertexCollection(34) = New VertexPositionColor(cubeB, cBlack)
            vertexCollection(35) = New VertexPositionColor(cubeF, cBlack)
        Else
            ' front 
            vertexCollection(0) = New VertexPositionColor(cubeA, cGreen)
            vertexCollection(1) = New VertexPositionColor(cubeB, cGreen)
            vertexCollection(2) = New VertexPositionColor(cubeC, cGreen)
            vertexCollection(3) = New VertexPositionColor(cubeB, cGreen)
            vertexCollection(4) = New VertexPositionColor(cubeD, cGreen)
            vertexCollection(5) = New VertexPositionColor(cubeC, cGreen)

            ' back  
            vertexCollection(6) = New VertexPositionColor(cubeG, cBlue)
            vertexCollection(7) = New VertexPositionColor(cubeF, cBlue)
            vertexCollection(8) = New VertexPositionColor(cubeE, cBlue)
            vertexCollection(9) = New VertexPositionColor(cubeH, cBlue)
            vertexCollection(10) = New VertexPositionColor(cubeF, cBlue)
            vertexCollection(11) = New VertexPositionColor(cubeG, cBlue)

            ' top 
            vertexCollection(12) = New VertexPositionColor(cubeE, cRed)
            vertexCollection(13) = New VertexPositionColor(cubeF, cRed)
            vertexCollection(14) = New VertexPositionColor(cubeA, cRed)
            vertexCollection(15) = New VertexPositionColor(cubeF, cRed)
            vertexCollection(16) = New VertexPositionColor(cubeB, cRed)
            vertexCollection(17) = New VertexPositionColor(cubeA, cRed)

            ' bottom  
            vertexCollection(18) = New VertexPositionColor(cubeH, cYellow)
            vertexCollection(19) = New VertexPositionColor(cubeG, cYellow)
            vertexCollection(20) = New VertexPositionColor(cubeC, cYellow)
            vertexCollection(21) = New VertexPositionColor(cubeD, cYellow)
            vertexCollection(22) = New VertexPositionColor(cubeH, cYellow)
            vertexCollection(23) = New VertexPositionColor(cubeC, cYellow)

            ' left 
            vertexCollection(24) = New VertexPositionColor(cubeC, cBlack)
            vertexCollection(25) = New VertexPositionColor(cubeG, cBlack)
            vertexCollection(26) = New VertexPositionColor(cubeA, cBlack)
            vertexCollection(27) = New VertexPositionColor(cubeA, cBlack)
            vertexCollection(28) = New VertexPositionColor(cubeG, cBlack)
            vertexCollection(29) = New VertexPositionColor(cubeE, cBlack)

            ' right  
            vertexCollection(30) = New VertexPositionColor(cubeH, cWhite)
            vertexCollection(31) = New VertexPositionColor(cubeD, cWhite)
            vertexCollection(32) = New VertexPositionColor(cubeB, cWhite)
            vertexCollection(33) = New VertexPositionColor(cubeH, cWhite)
            vertexCollection(34) = New VertexPositionColor(cubeB, cWhite)
            vertexCollection(35) = New VertexPositionColor(cubeF, cWhite)
        End If
        Dim vb = New VertexBuffer(resourceDevice, VertexPositionColor.VertexDeclaration, vertexCollection.Length, BufferUsage.WriteOnly)
        vb.SetData(0, vertexCollection, 0, vertexCollection.Length, 0)
        Return vb
    End Function
End Class
